import {
  __publicField,
  assertNever,
  init_buffer,
  init_process,
  typescript_exports
} from "./chunk-3S2LTSIS.js";

// src/context/FilePropertyMapper.ts
init_process();
init_buffer();
var FilePropertyMapper = class {
  constructor({ context }) {
    __publicField(this, "context");
    this.context = context;
  }
  getFilePropertyInfo({
    body,
    value
  }) {
    var _a;
    const result = {
      fileFields: [],
      bodyPropertyFields: []
    };
    const record = (_a = this.context.getRecord(value)) != null ? _a : {};
    for (const property of body.properties) {
      switch (property.type) {
        case "file":
          result.fileFields.push({
            name: this.context.getPropertyName(property.name),
            value: this.getSingleFileProperty({ property, record })
          });
          break;
        case "fileArray":
          result.fileFields.push({
            name: this.context.getPropertyName(property.name),
            value: this.getArrayFileProperty({ property, record })
          });
          break;
        case "bodyProperty":
          result.bodyPropertyFields.push({
            name: this.context.getPropertyName(property.name.name),
            value: this.getBodyProperty({ property, record })
          });
          break;
        default:
          assertNever(property);
      }
    }
    return result;
  }
  getSingleFileProperty({
    property,
    record
  }) {
    const fileValue = this.context.getSingleFileValue({ property, record });
    if (fileValue == null) {
      return typescript_exports.TypeLiteral.nop();
    }
    return typescript_exports.TypeLiteral.blob(fileValue);
  }
  getArrayFileProperty({
    property,
    record
  }) {
    const fileValues = this.context.getFileArrayValues({ property, record });
    if (fileValues == null) {
      return typescript_exports.TypeLiteral.nop();
    }
    return typescript_exports.TypeLiteral.array({ values: fileValues.map((value) => typescript_exports.TypeLiteral.blob(value)) });
  }
  getBodyProperty({
    property,
    record
  }) {
    const bodyPropertyValue = record[property.name.wireValue];
    if (bodyPropertyValue == null) {
      return typescript_exports.TypeLiteral.nop();
    }
    return this.context.dynamicTypeLiteralMapper.convert({
      typeReference: property.typeReference,
      value: bodyPropertyValue
    });
  }
};

export {
  FilePropertyMapper
};
